AutoCmdEvents = {
    'BUFADD': True,
    'BUFCREATE': True,
    'BUFDELETE': True,
    'BUFENTER': True,
    'BUFFILEPOST': True,
    'BUFFILEPRE': True,
    'BUFHIDDEN': True,
    'BUFLEAVE': True,
    'BUFNEW': True,
    'BUFNEWFILE': True,
    'BUFREAD': True,
    'BUFREADCMD': True,
    'BUFREADPOST': True,
    'BUFREADPRE': True,
    'BUFUNLOAD': True,
    'BUFWINENTER': True,
    'BUFWINLEAVE': True,
    'BUFWIPEOUT': True,
    'BUFWRITE': True,
    'BUFWRITECMD': True,
    'BUFWRITEPOST': True,
    'BUFWRITEPRE': True,
    'CMDUNDEFINED': True,
    'CMDWINENTER': True,
    'CMDWINLEAVE': True,
    'COLORSCHEME': True,
    'COMPLETEDONE': True,
    'CURSORHOLD': True,
    'CURSORHOLDI': True,
    'CURSORMOVED': True,
    'CURSORMOVEDI': True,
    'ENCODINGCHANGED': True,
    'FILEAPPENDCMD': True,
    'FILEAPPENDPOST': True,
    'FILEAPPENDPRE': True,
    'FILECHANGEDRO': True,
    'FILECHANGEDSHELL': True,
    'FILECHANGEDSHELLPOST': True,
    'FILEREADCMD': True,
    'FILEREADPOST': True,
    'FILEREADPRE': True,
    'FILETYPE': True,
    'FILEWRITECMD': True,
    'FILEWRITEPOST': True,
    'FILEWRITEPRE': True,
    'FILTERREADPOST': True,
    'FILTERREADPRE': True,
    'FILTERWRITEPOST': True,
    'FILTERWRITEPRE': True,
    'FOCUSGAINED': True,
    'FOCUSLOST': True,
    'FUNCUNDEFINED': True,
    'GUIENTER': True,
    'GUIFAILED': True,
    'INSERTCHANGE': True,
    'INSERTCHARPRE': True,
    'INSERTENTER': True,
    'INSERTLEAVE': True,
    'MENUPOPUP': True,
    'QUICKFIXCMDPOST': True,
    'QUICKFIXCMDPRE': True,
    'QUITPRE': True,
    'REMOTEREPLY': True,
    'SESSIONLOADPOST': True,
    'SHELLCMDPOST': True,
    'SHELLFILTERPOST': True,
    'SOURCECMD': True,
    'SOURCEPRE': True,
    'SPELLFILEMISSING': True,
    'STDINREADPOST': True,
    'STDINREADPRE': True,
    'SWAPEXISTS': True,
    'SYNTAX': True,
    'TABENTER': True,
    'TABLEAVE': True,
    'TERMCHANGED': True,
    'TERMRESPONSE': True,
    'TEXTCHANGED': True,
    'TEXTCHANGEDI': True,
    'USER': True,
    'VIMENTER': True,
    'VIMLEAVE': True,
    'VIMLEAVEPRE': True,
    'VIMRESIZED': True,
    'WINENTER': True,
    'WINLEAVE': True,
}
