BuiltinIdentifierMap = {
    'abs': True,
    'acos': True,
    'add': True,
    'append': True,
    'argc': True,
    'argidx': True,
    'argv': True,
    'asin': True,
    'atan': True,
    'atan2': True,
    'browse': True,
    'browsedir': True,
    'bufexists': True,
    'buflisted': True,
    'bufloaded': True,
    'bufname': True,
    'bufnr': True,
    'bufwinnr': True,
    'byte2line': True,
    'byteidx': True,
    'call': True,
    'ceil': True,
    'changenr': True,
    'char2nr': True,
    'cindent': True,
    'clearmatches': True,
    'col': True,
    'complete': True,
    'complete_add': True,
    'complete_check': True,
    'confirm': True,
    'copy': True,
    'cos': True,
    'cosh': True,
    'count': True,
    'cscope_connection': True,
    'cursor': True,
    'deepcopy': True,
    'delete': True,
    'did_filetype': True,
    'diff_filler': True,
    'diff_hlID': True,
    'empty': True,
    'escape': True,
    'eval': True,
    'eventhandler': True,
    'executable': True,
    'exists': True,
    'exp': True,
    'expand': True,
    'extend': True,
    'feedkeys': True,
    'filereadable': True,
    'filewritable': True,
    'filter': True,
    'finddir': True,
    'findfile': True,
    'float2nr': True,
    'floor': True,
    'fmod': True,
    'fnameescape': True,
    'fnamemodify': True,
    'foldclosed': True,
    'foldclosedend': True,
    'foldlevel': True,
    'foldtext': True,
    'foldtextresult': True,
    'foreground': True,
    'function': True,
    'garbagecollect': True,
    'get': True,
    'getbufline': True,
    'getbufvar': True,
    'getchar': True,
    'getcharmod': True,
    'getcmdline': True,
    'getcmdpos': True,
    'getcmdtype': True,
    'getcwd': True,
    'getfontname': True,
    'getfperm': True,
    'getfsize': True,
    'getftime': True,
    'getftype': True,
    'getline': True,
    'getloclist': True,
    'getmatches': True,
    'getpid': True,
    'getpos': True,
    'getqflist': True,
    'getreg': True,
    'getregtype': True,
    'gettabvar': True,
    'gettabwinvar': True,
    'getwinposx': True,
    'getwinposy': True,
    'getwinvar': True,
    'glob': True,
    'globpath': True,
    'has': True,
    'has_key': True,
    'haslocaldir': True,
    'hasmapto': True,
    'histadd': True,
    'histdel': True,
    'histget': True,
    'histnr': True,
    'hlID': True,
    'hlexists': True,
    'hostname': True,
    'iconv': True,
    'indent': True,
    'index': True,
    'input': True,
    'inputdialog': True,
    'inputlist': True,
    'inputrestore': True,
    'inputsave': True,
    'inputsecret': True,
    'insert': True,
    'isdirectory': True,
    'islocked': True,
    'items': True,
    'join': True,
    'keys': True,
    'len': True,
    'libcall': True,
    'libcallnr': True,
    'line': True,
    'line2byte': True,
    'lispindent': True,
    'localtime': True,
    'log': True,
    'log10': True,
    'map': True,
    'maparg': True,
    'mapcheck': True,
    'match': True,
    'matchadd': True,
    'matcharg': True,
    'matchdelete': True,
    'matchend': True,
    'matchlist': True,
    'matchstr': True,
    'max': True,
    'min': True,
    'mkdir': True,
    'mode': True,
    'nextnonblank': True,
    'nr2char': True,
    'pathshorten': True,
    'pow': True,
    'prevnonblank': True,
    'printf': True,
    'pumvisible': True,
    'range': True,
    'readfile': True,
    'reltime': True,
    'reltimestr': True,
    'remote_expr': True,
    'remote_foreground': True,
    'remote_peek': True,
    'remote_read': True,
    'remote_send': True,
    'remove': True,
    'rename': True,
    'repeat': True,
    'resolve': True,
    'reverse': True,
    'round': True,
    'search': True,
    'searchdecl': True,
    'searchpair': True,
    'searchpairpos': True,
    'searchpos': True,
    'server2client': True,
    'serverlist': True,
    'setbufvar': True,
    'setcmdpos': True,
    'setline': True,
    'setloclist': True,
    'setmatches': True,
    'setpos': True,
    'setqflist': True,
    'setreg': True,
    'settabvar': True,
    'settabwinvar': True,
    'setwinvar': True,
    'shellescape': True,
    'simplify': True,
    'sin': True,
    'sinh': True,
    'sort': True,
    'soundfold': True,
    'spellbadword': True,
    'spellsuggest': True,
    'split': True,
    'sqrt': True,
    'str2float': True,
    'str2nr': True,
    'strchars': True,
    'strdisplaywidth': True,
    'strftime': True,
    'stridx': True,
    'string': True,
    'strlen': True,
    'strpart': True,
    'strridx': True,
    'strtrans': True,
    'strwidth': True,
    'submatch': True,
    'substitute': True,
    'synID': True,
    'synIDattr': True,
    'synIDtrans': True,
    'synconcealed': True,
    'synstack': True,
    'system': True,
    'tabpagebuflist': True,
    'tabpagenr': True,
    'tabpagewinnr': True,
    'tagfiles': True,
    'taglist': True,
    'tan': True,
    'tanh': True,
    'tempname': True,
    'tolower': True,
    'toupper': True,
    'tr': True,
    'trunc': True,
    'type': True,
    'undofile': True,
    'undotree': True,
    'values': True,
    'virtcol': True,
    'visualmode': True,
    'winbufnr': True,
    'wincol': True,
    'winheight': True,
    'winline': True,
    'winnr': True,
    'winrestcmd': True,
    'winrestview': True,
    'winsaveview': True,
    'winwidth': True,
    'writefile': True,
}
